import { ChangeDetectorRef, EventEmitter, OnChanges } from '@angular/core';
import { FormGroup } from '@angular/forms';
import { Field, FieldConfigProperty, Form, FormConfig, Item, ItemConfig } from '@ec.components/core';
import { LoaderComponent } from '../loader/loader.component';
import { LoaderService } from '../loader/loader.service';
import { WithLoader } from '../loader/with-loader.interface';
import { Notification } from '../notifications/notification';
import { NotificationsService } from '../notifications/notifications.service';
import { WithNotifications } from '../notifications/with-notifications.interface';
import { SymbolService } from '../symbol/symbol.service';
import { FormService } from './form.service';
/** This component renders a form using a FieldConfig Object.
 *
 * Example:
 *
 * <example-url>https://components.entrecode.de/ui/form?e=1</example-url>
 */
export declare class FormComponent<T> implements OnChanges, WithLoader, WithNotifications {
    protected loaderService: LoaderService;
    protected notificationService: NotificationsService;
    formService: FormService;
    protected symbol: SymbolService;
    protected cdr: ChangeDetectorRef;
    /** The instance of Form that is used. */
    form: Form<T>;
    /** The current (angular) form group. */
    group: FormGroup;
    /** The current form config */
    config: FormConfig<T>;
    /** Recent Error notification */
    notifications: Notification[];
    /** You can also use a FormConfig/ItemConfig as input (with defined fields property) */
    configInput: FormConfig<T>;
    /** You can also use an Item as input */
    readonly item: Item<T>;
    /** If you pass an object to value, the form will generate an item from it. */
    value: T;
    /** If set to true, the form will be rendered empty, to be referenced from the outside. */
    empty: boolean;
    /** If set to false, the form will be rendered without a submit button. Default: true */
    submitButton: boolean;
    /** If true, no notifications will be emitted. */
    silent: boolean;
    /** The loader that should be used. */
    loader: LoaderComponent;
    /** Emits when the form is submitted. The form can only be submitted if all Validators succeeded. */
    submitted: EventEmitter<Form<T>>;
    /** Emits when the form has been initialized.  */
    ready: EventEmitter<FormComponent<T>>;
    /** Emits when a new instance of Form is present */
    changed: EventEmitter<FormComponent<T>>;
    /** debounce time till changed event/callback will be fired */
    debounceTime: number;
    /** If true, the form will only init once. On new changes, the form values will be patched (see patchValue) */
    lazy: boolean;
    /** The forms default loader. it is used when no loader is passed via the loader input */
    defaultLoader: LoaderComponent;
    /** The InputComponents that are used to control the fields */
    /** Injects the services. */
    constructor(loaderService: LoaderService, notificationService: NotificationsService, formService: FormService, symbol: SymbolService, cdr: ChangeDetectorRef);
    /** On change, the form instance is (re)created by combining all inputs.
     * If no item is given, an empty form is created using the config.
     * You can also pass just an item to use its config and body.*/
    ngOnChanges(changes?: any): void;
    /** Returns the column class for data-col, based on configured columns */
    getColumns(field: Field): string;
    patchValue(value?: T): void;
    /** Inits the form (if ready) */
    protected init(item?: Item<T>, config?: FormConfig<T>): void;
    /** Initializes the FormGroup which is generated from the current form instance. Sets valueChanges listener */
    protected initGroup(): void;
    patchObjectField(property: any, path: any, value: any): void;
    /** Adds a new field with the given config to the form */
    addField(property: string, config: FieldConfigProperty): void;
    /** Clears the current value */
    clear(): void;
    create(config?: ItemConfig<T>): void;
    /** edits a given Item instance by using its config and body. */
    edit(item: Item<T>): void;
    /** edits a given value by creating an item and calling edit. */
    editValue(value: T, config?: FormConfig<T>): void;
    /** Method that is invoked when the form is submitted.*/
    submit(): Promise<void>;
    /** Decides if the submit button should be rendered or not based on config */
    showSubmitButton(): boolean;
    /** Determindes if the given field's label should be shown based on hideFormLabel+hideFormLabelIfEmpty flags */
    showLabel(field: any, form: any): boolean;
    /** Determines if the field labels should always show the field property (EDITOR-431) */
    showTitle(field: any, form: any): boolean;
    /** Returns the current value of the form control group. When passing a property, it directly returns the property value. */
    getValue(property?: string): any;
    /** Returns an Error containing all field validator errors as subErrors.  */
    getErrors(formComponent: FormComponent<T>, touchedOnly?: boolean): Error;
}
